<?php

namespace Drupal\role_node_access\Plugin\Field\FieldWidget;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'role_node_access_widget' widget.
 *
 * @FieldWidget(
 *   id = "role_node_access_widget",
 *   module = "role_node_access",
 *   label = @Translation("Default"),
 *   field_types = {
 *     "role_node_access_type"
 *   }
 * )
 */
class RoleNodeAccessWidget extends WidgetBase implements ContainerFactoryPluginInterface {

  /**
   * The current user.
   */
  protected AccountInterface $user;

  /**
   * The entity_type.manager service.
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create(
      $container,
      $configuration,
      $plugin_id,
      $plugin_definition,
    );

    $instance->user = $container->get('current_user');
    $instance->entityTypeManager = $container->get('entity_type.manager');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\user\RoleInterface[] $allRoles */
    $allRoles = $this->entityTypeManager->getStorage('user_role')->loadMultiple();
    $displayedRoles = [];

    foreach ($allRoles as $rolename => $role) {
      $displayedRoles[$rolename] = $role->get('label');
    }

    return $element + [
      'value' => [
        '#type' => 'select',
        '#default_value' => $items[$delta]->value ?? NULL,
        '#options' => $displayedRoles,
        '#empty_value' => '',
      ],
    ];
  }

}
